=== modified file 'drizzled/serialize/table.proto'
--- drizzled/serialize/table.proto	2008-08-13 23:12:06 +0000
+++ drizzled/serialize/table.proto	2008-08-20 21:00:34 +0000
@@ -1,86 +1,148 @@
 package drizzle;
 
 message Table {
-  required string name = 1;
-  required string engine = 2;
-
-  enum FieldType {
-    DOUBLE = 0;
-    VARCHAR = 1;
-    TEXT = 2;
-    BLOB = 3;
-    ENUM = 4;
-    SET = 5;
-    TINYINT = 6;
-    SMALLINT = 7;
-    INTEGER = 8;
-    BIGINT = 9;
-    DECIMAL = 10;
-    VARBINARY = 11;
-    DATE = 12;
-    TIME = 13;
-    TIMESTAMP = 14;
-    DATETIME = 15;
+
+  enum TableType {
+    STANDARD = 0;
+    TEMPORARY = 1;
+  }
+
+  message StorageEngine {
+
+    message EngineOption {
+      enum EngineOptionType {
+        BOOL = 0;
+        INTEGER = 1;
+        STRING = 2;
+      }
+
+      required string name = 1;
+      required string value = 2;
+      required EngineOptionType type = 3;
+    }
+
+    required string name = 1;
+    repeated EngineOption option = 2;
+  }
+
+  message TableOptions {
+    optional uint64 auto_increment = 1;
+    optional string charset = 2;
+    optional string collation = 3;
+  }
+
+  message TableStats {
+    optional uint32 avg_row_length = 1;
+    optional uint64 max_rows = 2;
+    optional uint32 min_rows = 3;
+  }
+
+  message ForeignKeyConstraint {
+    required string name = 1;
+    required Field dependent = 2;
+    required Field parent = 3;
+    /** @TODO Finish this off... */
   }
 
   message Field {
+
+    enum FieldType {
+      DOUBLE = 0;
+      VARCHAR = 1;
+      TEXT = 2;
+      BLOB = 3;
+      ENUM = 4;
+      SET = 5;
+      TINYINT = 6;
+      SMALLINT = 7;
+      INTEGER = 8;
+      BIGINT = 9;
+      DECIMAL = 10;
+      VARBINARY = 11;
+      DATE = 12;
+      TIME = 13;
+      TIMESTAMP = 14;
+      DATETIME = 15;
+    }
+
+    message FieldOptions {
+      optional string default_value = 2;
+    }
+
+    message TimestampFieldOptions {
+      optional bool auto_updates = 1 [default = false];
+    }
+
+    message FieldConstraints {
+      required bool is_nullable = 1 [default = false];
+      optional bool is_unsigned = 2 [default = false];
+      repeated string expression = 16; /* Reserve 0-15 for frequenty accessed attributes */
+    }
+
+    message NumericFieldOptions {
+      optional bool is_autoincrement = 1 [default = false];
+      optional int32 length = 2;
+      optional int32 scale = 3;
+      optional int32 precision = 4;
+    }
+
+    message StringFieldOptions {
+      optional bool is_fixed_width = 1 [default = false];
+      optional int32 length = 2;
+      optional string charset = 3; /* Perhaps this can go away soon...*/
+      optional string collation = 4;
+    }
+
+    message SetFieldOptions {
+      required int32 count_elements = 1;
+      repeated string value = 2;
+    }
+
     required string name = 1;
     required FieldType type = 2;
-    optional string collation = 3;
-    optional string comment = 4;
-    optional bool unique = 5;
-    optional bool autoincrement = 6;
-    optional bool key = 7;
-    optional bool primary = 8;
-    optional bool is_unsigned = 9;
-    optional string custom_name = 10;
-    optional bool is_notnull = 11 [default = false];
-    optional int32 scale = 12;
-    optional string characterset = 13;
-    optional int32 length = 14;
-    optional string default_value = 15;
-    repeated string values = 16;
-    optional bool on_update = 17;
-  }
-
-  message KeyPart {
-    required string name = 1;
-    optional int32 length = 2;
-  }
-
-  enum IndexType {
-    ORDERED = 0;
-    HASH = 1;
+    optional FieldOptions options = 3;
+    optional FieldConstraints constraints = 4;
+    optional NumericFieldOptions numeric_options = 5;
+    optional StringFieldOptions string_options = 6;
+    optional string comment = 16; /* Reserve 0-15 for frequently accessed attributes */
+    optional SetFieldOptions set_options = 17;
+    optional TimestampFieldOptions timestamp_options = 18;
   }
 
   message Index {
+
+    enum IndexType {
+      UNKNOWN = 0;
+      HASH = 1;
+      BTREE = 2;
+      RTREE = 3;
+      FULLTEXT = 4;
+    }
+
+    message IndexPart {
+      required Field field = 1;
+      optional int32 compare_length = 2;
+      optional bool in_reverse_order = 3 [default = false];
+    }
+
     required string name = 1;
-    repeated KeyPart values = 2;
-    optional bool primary = 4;
-    optional int32 key_block_size = 5;
-    optional IndexType type = 6;
+    required bool is_primary = 2;
+    required bool is_unique = 3;
+    required IndexType type = 4 [default = UNKNOWN];
+    repeated IndexPart index_part = 5;
   }
 
-  repeated Field field = 4;
-  repeated Index index = 5;
-
-  optional string primary = 6;
-  optional int32  auto_increment = 7;
-  optional int32 avg_row_length = 8;
-  optional string character_set = 9;
-  optional bool checksum = 10;
-  optional string collation = 11;
-  optional string comment = 12;
-  optional string connection = 13;
-  optional string data_directory = 14;
-  optional string index_directory = 15;
-  optional bool delay_key_write = 16;
-  optional int32 max_rows = 17;
-  optional int32 min_rows = 18;
-  optional bool pack_keys = 19;
-  optional string row_format = 20;
-  optional string characterset = 21;
-  optional bool temp = 22;
+  required string name = 1;
+  required TableType type = 5;
+  required StorageEngine engine = 2;
+  repeated Field field = 3;
+  repeated Index index = 4;
+
+  repeated ForeignKeyConstraint fk_constraint = 8;
+  optional TableOptions options = 9;
+  optional TableStats stats = 10;
+
+  optional string comment = 16;
 }
 
 message TableList {

=== modified file 'drizzled/serialize/table_reader.cc'
--- drizzled/serialize/table_reader.cc	2008-08-13 23:12:06 +0000
+++ drizzled/serialize/table_reader.cc	2008-08-20 21:00:34 +0000
@@ -8,30 +8,32 @@
   Written from Google proto example
 */
 
-void printType(const drizzle::Table::Field *field) 
+void print_field(const drizzle::Table::Field *field) 
 {
+  using namespace drizzle;
+  cout << "\t`" << field->name() << "`";
   switch (field->type())
   {
-  case drizzle::Table::DOUBLE:
+    case Table::Field::DOUBLE:
     cout << " DOUBLE ";
     break;
-  case drizzle::Table::VARCHAR:
-    cout << " VARCHAR(" << field->length() << ")";
+  case Table::Field::VARCHAR:
+    cout << " VARCHAR(" << field->string_options().length() << ")";
     break;
-  case drizzle::Table::TEXT:
+  case Table::Field::TEXT:
     cout << " TEXT ";
     break;
-  case drizzle::Table::BLOB:
+  case Table::Field::BLOB:
     cout << " BLOB ";
     break;
-  case drizzle::Table::ENUM:
+  case Table::Field::ENUM:
     {
       int x;
 
       cout << " ENUM(";
-      for (x= 0; x < field->values_size() ; x++)
+      for (x= 0; x < field->set_options().value_size() ; x++)
       {
-        const string type= field->values(x);
+        const string type= field->set_options().value(x);
 
         if (x != 0)
           cout << ",";
@@ -40,126 +42,176 @@
       cout << ") ";
       break;
     }
-  case drizzle::Table::SET:
+  case Table::Field::SET:
     cout << " SET ";
     break;
-  case drizzle::Table::TINYINT:
-    cout << " TINYINNT ";
+  case Table::Field::TINYINT:
+    cout << " TINYINT ";
     break;
-  case drizzle::Table::SMALLINT:
+  case Table::Field::SMALLINT:
     cout << " SMALLINT ";
     break;
-  case drizzle::Table::INTEGER:
+  case Table::Field::INTEGER:
     cout << " INTEGER ";
     break;
-  case drizzle::Table::BIGINT:
+  case Table::Field::BIGINT:
     cout << " BIGINT ";
     break;
-  case drizzle::Table::DECIMAL:
-    cout << " DECIMAL(" << field->length() << "," << field->scale() << ") ";
-    break;
-  case drizzle::Table::VARBINARY:
-    cout << " VARBINARY(" << field->length() << ") ";
-    break;
-  case drizzle::Table::DATE:
+  case Table::Field::DECIMAL:
+    cout << " DECIMAL(" << field->numeric_options().precision() << "," << field->numeric_options().scale() << ") ";
+    break;
+  case Table::Field::VARBINARY:
+    cout << " VARBINARY(" << field->string_options().length() << ") ";
+    break;
+  case Table::Field::DATE:
     cout << " DATE ";
     break;
-  case drizzle::Table::TIME:
+  case Table::Field::TIME:
     cout << " TIME ";
     break;
-  case drizzle::Table::TIMESTAMP:
+  case Table::Field::TIMESTAMP:
     cout << " TIMESTAMP ";
     break;
-  case drizzle::Table::DATETIME:
+  case Table::Field::DATETIME:
     cout << " DATETIME ";
     break;
   }
 
-  if (field->has_characterset())
-    cout << " CHARACTER SET " << field->characterset();
-
-  if (field->has_collation())
-    cout << " COLLATE " << field->collation();
-
-  if (field->is_notnull())
+  if (field->type() == Table::Field::INTEGER
+      || field->type() == Table::Field::BIGINT
+      || field->type() == Table::Field::SMALLINT
+      || field->type() == Table::Field::TINYINT) {
+    if (field->has_constraints()
+        && field->constraints().has_is_unsigned())
+      if (field->constraints().is_unsigned())
+        cout << " UNSIGNED";
+
+    if (field->has_numeric_options() &&
+      field->numeric_options().is_autoincrement())
+      cout << " AUTOINCREMENT ";
+  }
+
+  if (! field->has_constraints()
+      && field->constraints().is_nullable())
     cout << " NOT NULL ";
 
-  if (field->has_default_value())
-    cout << " DEFAULT `" << field->default_value() << "` " ;
-
-  if (field->on_update())
-    cout << " ON UPDATE CURRENT_TIMESTAMP";
-
-  if (field->autoincrement())
-    cout << " AUTOINCREMENT ";
+  if (field->type() == Table::Field::TEXT
+      || field->type() == Table::Field::VARCHAR) {
+    if (field->string_options().has_charset())
+      cout << " CHARACTER SET " << field->string_options().charset();
+
+    if (field->string_options().has_collation())
+      cout << " COLLATE " << field->string_options().collation();
+  }
+
+  if (field->options().has_default_value())
+    cout << " DEFAULT `" << field->options().default_value() << "` " ;
+
+  if (field->type() == Table::Field::TIMESTAMP)
+    if (field->timestamp_options().has_auto_updates()
+      && field->timestamp_options().auto_updates())
+      cout << " ON UPDATE CURRENT_TIMESTAMP";
 
   if (field->has_comment())
     cout << " COMMENT `" << field->comment() << "` ";
 }
 
-void printTable(const drizzle::Table *table) 
+void print_engine(const drizzle::Table::StorageEngine *engine) {
+  using namespace drizzle;
+  uint32_t x;
+
+  cout << " ENGINE = " << engine->name()  << endl;
+
+  for (x= 0; x < engine->option_size(); ++x) {
+    const Table::StorageEngine::EngineOption option= engine->option(x);
+    cout << "\t" << option.name() << " = " << option.value() << endl;
+  }
+}
+
+void print_index(const drizzle::Table::Index *index) {
+  using namespace drizzle;
+  uint32_t x;
+
+  if (index->is_primary())
+    cout << " PRIMARY"; 
+  else if (index->is_unique())
+    cout << " UNIQUE"; 
+  cout << " KEY `" << index->name() << "` (";
+  {
+    int x;
+
+    for (x= 0; x < index->index_part_size() ; x++)
+    {
+      const Table::Index::IndexPart part= index->index_part(x);
+
+      if (x != 0)
+        cout << ",";
+      cout << "`" << part.field().name() << "`";
+      if (part.has_compare_length())
+        cout << "(" << part.compare_length() << ")";
+    }
+    cout << ")";
+  }
+  cout << "\t";
+}
+
+void print_table(const drizzle::Table *table) 
 {
+  using namespace drizzle;
   uint32_t x;
 
-  cout << "CREATE TABLE";
-
-  if (table->temp())
-    cout << " TEMPORARY";
-
-  cout << " `" << table->name() << "` (" << endl;
+  cout << "CREATE ";
+
+  if (table->type() == Table::TEMPORARY)
+    cout << "TEMPORARY ";
+
+  cout << "TABLE `" << table->name() << "` (" << endl;
   
   for (x= 0; x < table->field_size() ; x++)
   {
-    const drizzle::Table::Field field = table->field(x);
+    const Table::Field field = table->field(x);
+
+    if (x != 0)
+      cout << "," << endl;
+
+    print_field(&field);
+  }
+
+  for (x= 0; x < table->index_size() ; x++)
+  {
+    const Table::Index index= table->index(x);
 
     if (x != 0)
       cout << "," << endl;;
 
-    cout << "\t`" << field.name() << "`";
-    printType(&field);
-  }
-
-  for (x= 0; x < table->index_size() ; x++)
-  {
-    const drizzle::Table::Index index = table->index(x);
-
-    cout << "," << endl;;
-
-    cout << "\t";
-
-    if (index.primary())
-      cout << " PRIMARY KEY (`" << index.name() << "`)";
-    else
-    {
-      int x;
-
-      cout << " UNIQUE KEY `" << index.name() << "` (";
-      for (x= 0; x < index.values_size() ; x++)
-      {
-        const drizzle::Table::KeyPart key= index.values(x);
-
-        if (x != 0)
-          cout << ",";
-        cout << "`" << key.name() << "`";
-      }
-      cout << ")";
-    }
+    print_index(&index);
+
   }
   cout << endl;
 
   cout << ") " << endl;
-  if (table->has_collation())
-    cout << " COLLATE = `" << table->collation() << "` " << endl;;
-  if (table->has_characterset())
-    cout << " CHARACTER SET = `" << table->characterset() << "` " << endl;;
+  
+  print_engine(&table->engine());
+
+  /*
+  if (table->has_options())
+    print_table_options(&table->options());
+  if (table->has_stats())
+    print_table_stats(&table->stats());
+  */
   if (table->has_comment())
-    cout << " COMMENT = `" << table->comment() << "` " << endl;;
-  if (table->has_engine())
-  if (table->has_data_directory())
-    cout << " DATA DIRECTORY = `" << table->data_directory() << "` " << endl;;
-  if (table->has_index_directory())
-    cout << " INDEX DIRECTORY = `" << table->index_directory() << "`" << endl;
-  cout << " ENGINE = " << table->engine() << ";"  << endl;
+    cout << " COMMENT = `" << table->comment() << "` " << endl;
+}
+
+void print_table_stats(const drizzle::Table::TableStats *stats) {
+  
+}
+
+void print_table_options(const drizzle::Table::TableOptions *options) {
+  if (options->has_collation())
+    cout << " COLLATE = `" << options->collation() << "` " << endl;
+  if (options->has_charset())
+    cout << " CHARACTER SET = `" << options->charset() << "` " << endl;
 }
 
 int main(int argc, char* argv[]) 
@@ -183,7 +235,7 @@
     }
   }
 
-  printTable(&table);
+  print_table(&table);
 
   return 0;
 }

=== modified file 'drizzled/serialize/table_writer.cc'
--- drizzled/serialize/table_writer.cc	2008-08-13 23:12:06 +0000
+++ drizzled/serialize/table_writer.cc	2008-08-20 21:00:34 +0000
@@ -8,90 +8,185 @@
   Written from Google proto example
 */
 
-void updateTable(drizzle::Table *table, const char *name) 
+void fill_engine(drizzle::Table::StorageEngine *engine) {
+  using namespace drizzle;
+  using std::string;
+  int16_t x;
+
+  engine->set_name("InnoDB");
+  Table::StorageEngine::EngineOption *option;
+
+  string option_names[2]= {
+    "INDEX_DIRECTORY"
+    , "DATA_DIRECTORY"
+  };
+
+  string option_values[2]= {
+    "/var/drizzle/indexdir"
+    , "/var/drizzle/datadir"
+  };
+
+  /* Add some engine options */
+  for (x= 0; x < 2; ++x) {
+    option= engine->add_option();
+    option->set_name(option_names[x]);
+    option->set_value(option_values[x]);
+    option->set_type(Table::StorageEngine::EngineOption::STRING);
+  }
+}
+
+void new_index_to_table(
+    drizzle::Table *table
+    , const std::string name
+    , uint16_t num_index_parts
+    , uint32_t field_indexes[]
+    , uint32_t compare_lengths[]
+    , bool is_primary
+    , bool is_unique
+    ) {
+  using namespace drizzle;
+  uint16_t x;
+
+  Table::Index *index;
+  Table::Field *field;
+  Table::Index::IndexPart *index_part;
+
+  index= table->add_index();
+
+  index->set_name(name);
+  index->set_type(Table::Index::BTREE);
+  index->set_is_primary(is_primary);
+  index->set_is_unique(is_unique);
+
+  while (x < num_index_parts) {
+
+    index_part= index->add_index_part();
+
+    field= index_part->mutable_field();
+    *field= table->field(field_indexes[x]);
+
+    if (compare_lengths[x] > 0)
+      index_part->set_compare_length(compare_lengths[x]);
+
+    x++;
+  }
+}
+
+void fill_table(drizzle::Table *table, const char *name) 
 {
-  int x;
+  uint16_t x;
+
+  using namespace drizzle;
+
+  Table::Field *field;
+  Table::Field::FieldConstraints *field_constraints;
+  Table::Field::FieldOptions *field_options;
+  Table::Field::StringFieldOptions *string_field_options;
+  Table::Field::NumericFieldOptions *numeric_field_options;
+  Table::Field::SetFieldOptions *set_field_options;
 
   table->set_name(name);
-  table->set_engine("Innodb");
+  table->set_type(Table::STANDARD);
 
   /* Write out some random varchar */
   for (x= 0; x < 3; x++)
   {
     char buffer[1024];
-    drizzle::Table::Field *field = table->add_field();
+    field= table->add_field();
+    field_constraints= field->mutable_constraints();
+    string_field_options= field->mutable_string_options();
 
     sprintf(buffer, "sample%u", x);
 
     field->set_name(buffer);
-    field->set_type(drizzle::Table::VARCHAR);
-    if (x % 2)
-      field->set_is_notnull(true);
-
-    field->set_length(rand() % 100);
+    field->set_type(Table::Field::VARCHAR);
+
+    field_constraints->set_is_nullable((x % 2));
+
+    string_field_options->set_length(rand() % 100);
 
     if (x % 3)
     {
-      field->set_collation("utf8_swedish_ci");
-      field->set_characterset("utf8");
+      string_field_options->set_collation("utf8_swedish_ci");
+      string_field_options->set_charset("utf8");
     }
   }
 
   /* Write out an INTEGER */
   {
-    drizzle::Table::Field *field = table->add_field();
+    field= table->add_field();
     field->set_name("number");
-    field->set_type(drizzle::Table::INTEGER);
+    field->set_type(Table::Field::INTEGER);
   }
   /* Write out a ENUM */
   {
-    drizzle::Table::Field *field = table->add_field();
-    field->set_type(drizzle::Table::ENUM);
+    field= table->add_field();
+    field->set_type(Table::Field::ENUM);
     field->set_name("colors");
-    field->add_values("red");
-    field->add_values("blue");
-    field->add_values("green");
+
+    set_field_options= field->mutable_set_options();
+    set_field_options->add_value("red");
+    set_field_options->add_value("blue");
+    set_field_options->add_value("green");
+    set_field_options->set_count_elements(set_field_options->value_size());
   }
   /* Write out a BLOB */
   {
-    drizzle::Table::Field *field = table->add_field();
+    field= table->add_field();
     field->set_name("some_btye_string");
-    field->set_type(drizzle::Table::BLOB);
+    field->set_type(Table::Field::BLOB);
   }
   /* Write out a DECIMAL */
   {
-    drizzle::Table::Field *field = table->add_field();
+    field= table->add_field();
     field->set_name("important_number");
-    field->set_type(drizzle::Table::DECIMAL);
-    field->set_length(8);
-    field->set_scale(3);
-  }
-  /* Write out a VARCHAR with index */
-  {
-    drizzle::Table::Field *field = table->add_field();
-    field->set_name("important_string");
-    field->set_type(drizzle::Table::VARCHAR);
-    field->set_length(20);
-    field->set_unique(true);
-  }
-
-  {
-    drizzle::Table::Index *index = table->add_index();
-    index->set_name("number");
-    index->set_primary(true);
-  }
-
-  for (x= 0; x < 2; x++)
-  {
-    char buffer[1024];
-    drizzle::Table::Index *index = table->add_index();
-    drizzle::Table::KeyPart *keypart = index->add_values();
-
-    sprintf(buffer, "sample%u", x);
-    index->set_name(buffer);
-
-    keypart->set_name(buffer);
-  }
+    field->set_type(Table::Field::DECIMAL);
+
+    field_constraints= field->mutable_constraints();
+    field_constraints->set_is_nullable(true);
+    
+    numeric_field_options= field->mutable_numeric_options();
+    numeric_field_options->set_precision(8);
+    numeric_field_options->set_scale(3);
+  }
+
+  {
+  uint32_t fields_in_index[1]= {6};
+  uint32_t compare_lengths_in_index[1]= {0};
+  bool is_unique= true;
+  bool is_primary= false;
+  /* Add a single-column index on important_number field */
+  new_index_to_table(
+      table
+    , "idx_important_decimal"
+    , 1
+    , fields_in_index
+    , compare_lengths_in_index
+    , is_primary
+    , is_unique
+    );
+  }
+
+  {
+  /* Add a double-column index on first two varchar fields */
+  uint32_t fields_in_index[2]= {0,1};
+  uint32_t compare_lengths_in_index[2]= {20,35};
+  bool is_unique= true;
+  bool is_primary= true;
+  new_index_to_table(
+      table
+    , "idx_varchar1_2"
+    , 2
+    , fields_in_index
+    , compare_lengths_in_index
+    , is_primary
+    , is_unique
+    );
+  }
+
+  /* Do engine-specific stuff */
+  Table::StorageEngine *engine= table->mutable_engine();
+  fill_engine(engine);
 
 }
 
@@ -106,7 +201,7 @@
 
   drizzle::Table table;
 
-  updateTable(&table, "example_table");
+  fill_table(&table, "example_table");
 
   fstream output(argv[1], ios::out | ios::trunc | ios::binary);
   if (!table.SerializeToOstream(&output)) 

